######################################################################
#
# File: 	PyDebugGdbChild.py
#
# Purpose:	This is a derived class for spawning a child process
#		with a filter specifically for PyDebug
#
# Version:	$Id: PyDebugGdbChild.py,v 1.3 2001/05/22 19:17:51 pstoltz Exp $
#
# Copyright 2000 Tech-X Corporation, all rights reserved
#
######################################################################

from txChild import *
##########
#
# This is a derived class to provide a child process specifically 
# for PyDebug with a built-in filter on the output pipe.
# The child process for PyDebug is some compiled-language debugger,
# like gdb.
#
##########
class PyDebugGdbChild (txChild):

  # This method is the constructor for PyDebugGdbChild.
  #
  # **Arguments**
  #
  # *proc_name* a string containing the name of the debugger (e.g. gdb)
  #
  # *app_name* a string containing the name of the application to
  # be debugged (e.g. python)
  #
  # *breakcmds* a dictionary of break commands and associated on/off flags
  #
  # **returns** none
  def __init__(self,proc_name,app_name,breakcmds):
    txChild.__init__(self,proc_name,app_name)
    self.breakcmds=breakcmds

  ##########
  #
  # This derived method listens to output from the child and filters it, 
  # and sends the screen if appropriate
  #
  # **Arguments**
  #
  # *istr* A pipe attached to the child process (the compiled-language
  # debugger in this case)
  #
  # *myLock* A thread lock (in case reading from a file)
  #
  # **returns** none
  #
  ##########
  def transferIstr(self,istr,myLock):

    # This string keeps track of matches
    mString = ""
    # This string accumulates so we can swallow whole lines
    aString = ""

    # These are flags to test what kind of match we have
    swallow=0
    lockthread=0
    breakmatch=0
    prompt=0
  
    # These are lists of the different kind of strings to match
    lockList = []
    breakList = []
    promptList = []
      # But we may want to swallow more than one line
      # so let's keep track of how many lines to swallow.
      # 
      # That means we need a Dictionary
    swallowDict = {}
  
    # Add to each list
    # These are all the prompts (i.e. no carriage return follows)
    promptList.append(">>> ")
    promptList.append("PYDEBUG>> ")
    promptList.append("(gdb) ")
    promptList.append("(y or n) ")
    promptList.append("quit---")
  
    # These are when a break point is reached and we may want to lock
    lockList.append("Breakpoint 1,")
    lockList.append("Breakpoint 2,")
    lockList.append("Breakpoint 3,")
  
    # These are when a breakpoint is set and we want to toggle its flag
    breakList.append("Breakpoint 1 ")
    breakList.append("Breakpoint 2 ")
    breakList.append("Breakpoint 3 ")
  
    # These lines are the beginnings of something we'd like to swallow.  
    # The number is how many lines to swallow.
    swallowDict["No source file named "]=1
    swallowDict["Note: breakpoint "]=1
    swallowDict["Program received signal SIGINT"]=3
  
    # Read all the time
    while 1:
      try:
        appchar = istr.read(1)    # This blocks this thread until it gets a char
        # 
        # This is John's old trick of building up a line by adding a single char
        # mString will get reset any time we don't match
        # aString accumulates until a carriage return or prompt
        mString = mString + appchar
        aString = aString + appchar
      except IOError:
        sys.stdout.write("IOError: transferIstr returning.\n")
        sys.stdout.flush()
        return
  
      # Check for matching
      # If whole match string is matched, set the flag
      matched=0
  
      for mstr in promptList:
        if len(mString) and (len(mString) <= len(mstr)):
          if appchar==mstr[len(mString)-1]:
            matched = 1
            if mstr == mString:
              prompt=1
  
      for mstr in swallowDict.keys():
        if len(mString) and (len(mString) <= len(mstr)):
          if appchar==mstr[len(mString)-1]:
            matched = 1
            if mstr == mString:
              swallow = swallowDict[mstr]
  
      for mstr in breakList:
        if len(mString) and (len(mString) <= len(mstr)):
          if appchar==mstr[len(mString)-1]:
            matched = 1
            if mstr == mString:
              breakmatch = 1
  
      for mstr in lockList:
        if len(mString) and (len(mString) <= len(mstr)):
          if appchar==mstr[len(mString)-1]:
            matched = 1
            if mstr == mString:
              lockthread = 1
      
      # Reset mString if not matched
      if not matched:
        mString = ""
  
      # If any flag is set, do what is appropriate
      elif (breakmatch+prompt+swallow+lockthread):
  
        # if you've matched "Breakpoint X ", turn off 
        # that break so it isn't set again
        if breakmatch:
          breakmatch=0
          which_bpoint=int(mString[len(mString)-2])
          self.breakcmds[which_bpoint]=0
  
        # if you've matched "Breakpoint, " acquire lock
        # this will stop any reading from a python script file
        # while gdb is in charge
        if lockthread:
          lockthread=0
          myLock.acquire()
  
        # if you've matched a prompt, write it
        # because you aren't going to get a carriage return
        if prompt:
          prompt=0
          if swallow:
            swallow=swallow-1
            aString=""
          sys.stdout.write(aString)
          sys.stdout.flush()
          aString=""
  
        # Finally, reset mString if done with swallows:
        if not swallow:
          mString=""
  
      # Now, check if you've hit a carriage return
      # You need to do this matched or not
      if len(aString) and aString[len(aString)-1]=="\n":
          if swallow: 
            # Here we set flag back to zero
            # and set aString to nothing
            swallow=swallow-1
            aString=""
          sys.stdout.write(aString)
          sys.stdout.flush()
          aString=""

