######################################################################
#
# File: 	PyDebugGdbCmd.py
#
# Purpose:	A derived class specific to PyDebug.
#		This filters input from stdin, looking
#		for special commands (e.g. gdb commands)
#
# Version:	$Id: PyDebugGdbCmd.py,v 1.1 2001/05/22 19:17:51 pstoltz Exp $
#
# Copyright 2000 Tech-X Corporation, all rights reserved
#
######################################################################

__version__ = "Peter Stoltz, May 2001"

from cmd import *
import sys

##########
#
# This is a derived class to filter lines for the PyDebug application.
# It includes things specific to PyDebug, like a dictionary of breakcmds.
#
##########
#class PyDebugGdbCmd(txCmd):
class PyDebugGdbCmd(Cmd):

  # This is the constructor method.
  # It stores the breakcmds dictionary as member data.
  # **Arguments**
  #
  # *ostr* A pipe to which txCmd sends filtered output 
  #
  # *breakcmds* A dictionary of breaks cataloged by number that is shared 
  # with the other thread.  The break number is a key with the other member 
  # a status (either enabled or disabled).
  #
  # **returns** none
  #
  def __init__(self, ostr, breakcmds):
    Cmd.__init__(self)
    self.ostr= ostr
    # Breakcmds needs to be shared between both threads
    self.breakcmds = breakcmds
    # Breakcmds only keeps track by number
    # Breaklist keeps track of actual string command
    self.breaklist = []
    self.prompt = ""

  def help_exit(self):
    print "Quits the application."
  def do_exit(self, line):
    print "PyDebug quitting..."
    sys.exit(0)

  def help_quit(self):
    print "Quits the application."
  def do_quit(self, line):
    print "PyDebug quitting..."
    sys.exit(0)

  def help_EOF(self):
    print "Quits the application."
  def do_EOF(self, line):
    print 
    print "PyDebug quitting..."
    sys.exit(0)

  def help_import(self):
    print "Checks on breakpoints after importing."
  def do_import(self, line):
    # First, do the import so the needed files are present
    self.ostr.write("import "+line + "\n")
    self.ostr.flush()
    # Now check if there are any breaks to try
    # by summing up all breakcmd flags (if any)
    if len(self.breakcmds):
      tot=0
      for brk in self.breakcmds.keys():
        tot=tot+self.breakcmds[brk]
      if tot:
        self.ostr.write("import os\n")
        self.ostr.write("mypid=os.getpid()\n")
        self.ostr.write("os.kill(mypid,2)\n")
        for brk in self.breakcmds.keys():
          if self.breakcmds[brk]:
            # Again, the indexing is off by 1...sigh
            which_cmd = int(brk)-1
            self.ostr.write(self.breaklist[which_cmd]+"\n")
            print self.breaklist[which_cmd]
        self.ostr.write("cont\n")

  def help_break(self):
    print "Adds a breakpoint only if it isn't already in list."
  def do_break(self, line):
    if line not in self.breaklist:
      self.breaklist.append("break "+line)
      # Add the break number to the breakcmds dictionary
      # and set its flag to 1 (break is enabled)
      # Unfortunately, breaks start at 1 so this is not elegant
      cmd_num = len(self.breakcmds)+1
      self.breakcmds[cmd_num]=1
      # Write a message so user knows break is set
      sys.stdout.write("PyDebug breakpoint "+str(cmd_num)+" set\n")
      sys.stdout.flush()
      self.ostr.write("\n")
      self.ostr.flush()

  def help_run(self):
    print "Checks if any breakpoints need to be deleted before issuing 'run'"
  def do_run(self, line):
    if len(self.breakcmds):
      # First, toggle off confirmation
      # Then delete all breakpoints
      # Then toggle back on confirmation
      self.ostr.write("set confirm off\n")
      self.ostr.write("d\n")
      self.ostr.write("set confirm on\n")
      # Now turn back on all breakpoints
      for brk in self.breakcmds.keys():
        self.breakcmds[brk]=1
    # Finally, issue 'run' command
    self.ostr.write("run\n")

  def help_debug(self):
    print "Sets up some debugging."
  def do_debug(self, line):
    print "PyDebug is doing some debugging..."
    self.ostr.write("\n")

  def help__gdb_(self):
    print "This is _gdb_ help"
  def do__gdb_(self, line):
    self.ostr.write("import os\n")
    self.ostr.write("mypid=os.getpid()\n")
    self.ostr.write("os.kill(mypid,2)\n")
    self.ostr.write(line + "\n")
    self.ostr.write("cont\n")

  def do_help(self, arg):
    Cmd.do_help(self, arg)
    self.ostr.write("\n")

  def emptyline(self):
    self.ostr.write("\n")

  def default(self, line):
    self.ostr.write(line + "\n")

  def preloop(self):
    pass

  def postcmd(self, stop, line):
    return stop

  def precmd(self, line):
    if line and line[0] == '$': 
      line='_gdb_ '+line[1:]
    return line
